@@ -4,7 +4,6 @@ import collections
 import operator
 from functools import partial
 from itertools import filterfalse, zip_longest
-from random import Random
 from collections.abc import Sequence
 from toolz.utils import no_default
 
@@ -149,8 +148,7 @@ def _merge_sorted_binary(seqs):
     try:
         val2 = next(seq2)
     except StopIteration:
-        for val1 in seq1:
-            yield val1
+        yield from seq1
         return
 
     for val1 in seq1:
@@ -168,12 +166,10 @@ def _merge_sorted_binary(seqs):
             yield val1
     else:
         yield val2
-        for val2 in seq2:
-            yield val2
+        yield from seq2
         return
     yield val1
-    for val1 in seq1:
-        yield val1
+    yield from seq1
 
 
 def _merge_sorted_binary_key(seqs, key):
@@ -192,8 +188,7 @@ def _merge_sorted_binary_key(seqs, key):
     try:
         val2 = next(seq2)
     except StopIteration:
-        for val1 in seq1:
-            yield val1
+        yield from seq1
         return
     key2 = key(val2)
 
@@ -214,12 +209,10 @@ def _merge_sorted_binary_key(seqs, key):
             yield val1
     else:
         yield val2
-        for val2 in seq2:
-            yield val2
+        yield from seq2
         return
     yield val1
-    for val1 in seq1:
-        yield val1
+    yield from seq1
 
 
 def interleave(seqs):
@@ -733,7 +726,12 @@ def partition_all(n, seq):
         try:
             # If seq defines __len__, then
             # we can quickly calculate where no_pad starts
-            yield prev[:len(seq) % n]
+            end = len(seq) % n
+            if prev[end - 1] is no_pad or prev[end] is not no_pad:
+                raise LookupError(
+                    'The sequence passed to `parition_all` has invalid length'
+                )
+            yield prev[:end]
         except TypeError:
             # Get first index of no_pad without using .index()
             # https://github.com/pytoolz/toolz/issues/387
@@ -818,6 +816,7 @@ def join(leftkey, leftseq, rightkey, rightseq,
     This is a semi-streaming operation.  The LEFT sequence is fully evaluated
     and placed into memory.  The RIGHT sequence is evaluated lazily and so can
     be arbitrarily large.
+
     (Note: If right_default is defined, then unique keys of rightseq
         will also be stored in memory.)
 
@@ -829,7 +828,7 @@ def join(leftkey, leftseq, rightkey, rightseq,
 
     >>> cities = [('Alice', 'NYC'),
     ...           ('Alice', 'Chicago'),
-    ...           ('Dan', 'Syndey'),
+    ...           ('Dan', 'Sydney'),
     ...           ('Edith', 'Paris'),
     ...           ('Edith', 'Berlin'),
     ...           ('Zhao', 'Shanghai')]
@@ -1052,5 +1051,7 @@ def random_sample(prob, seq, random_state=None):
     [7, 9, 19, 25, 30, 32, 34, 48, 59, 60, 81, 98]
     """
     if not hasattr(random_state, 'random'):
+        from random import Random
+
         random_state = Random(random_state)
     return filter(lambda _: random_state.random() < prob, seq)
